

#include <stdio.h>
#include "stack.h"
#include "requestHandler.h"
#include "threads.h"
#include "os_socket.h"

class ClientConnection : public TObject
{
  public:
  struct sockaddr_in remote_addr;
  int fd_size;
  int new_socket;
  char IP[16];
  SktStack* stack;
  int sockfd;
  ClientConnection(int sockfd):sockfd(sockfd)
  {
    new_socket = 0;
    fd_size = sizeof(struct sockaddr_in);
    stack = new SktStack();
  }

  ~ClientConnection()
  {
     delete stack;
  }

  int accept()
  {
     new_socket = os_socket_accept(sockfd,(sockaddr*)&remote_addr,&fd_size);
     #ifdef _WIN32
      snprintf(IP,16,"%d.%d.%d.%d",
                 remote_addr.sin_addr.S_un.S_un_b.s_b1,
                 remote_addr.sin_addr.S_un.S_un_b.s_b2,
                 remote_addr.sin_addr.S_un.S_un_b.s_b3,
                 remote_addr.sin_addr.S_un.S_un_b.s_b4);
      IP[15] = 0;
     #else
      IP[0 ] = 0;
     #endif
     return new_socket;
  }
};


class HandlerThread : public TThread
{

  public:
   ClientConnection *connection;

  void execute()
  {
    printf("%s : started\n",getName());
    RequestHandler* myhandler = new RequestHandler(); 
    printf("%s: Client [%s] connected\n",getName(),connection->IP);

    int res = 0;
       while(connection->new_socket)
       {
          res = myhandler->handleRequest(connection->new_socket,connection->stack);
          if (res != SktStack::STACK_OK)
          {
            printf("%s: Request from client [%s] failed with error: %s\n",getName(),connection->IP,SktStack::errorMessage(res));
            if (res == SktStack::STACK_RECV_HEADER_ERROR) printf("%s: Probably client [%s] dropped the connection\n",getName(),connection->IP);
            os_socket_close(connection->new_socket);
            connection->new_socket = 0;
          }
          else
          {
             // request done with OK.
          }
       }//while connected

    printf("%s: Client [%s] disconnected\n",getName(),connection->IP);

    delete connection;
    delete myhandler;
  }//execute
};

class MainThread : public TThread
{

 public:
 int port;
 bool running;
 void execute()
 {
      SOCKET sockfd = os_socket_server(port);

	if (sockfd == -1)
	  {
	    printf("Couldn't create socket for port %d\n",port);
            return;
	  }
	else
	  {
	    printf("Listening at port %d ...\n",port);
	  }


      ClientConnection* connection;
      HandlerThread*    thread;

      running = true;
      
      while (running)
      {
        connection = new ClientConnection(sockfd);
        connection->setName("Client Connection");
        if (connection->accept() != 0)
        {
          printf("creating thread...\n");
          thread = new HandlerThread();
          thread->setName("Client thread");
          thread->connection = connection;
          // thread will delete connection
          thread->start();
        }
        else
        {
           delete connection;
        }
      }

  }
};


int main(int argc, char** argv)
{
  printf("SQLite Server (prototype) Version 0.31\n");

  if (argc != 2)
  {
      printf("Usage : %s <Port>\n",argv[0]);
  }
  else
  {
      int port;
      sscanf(argv[1],"%d",&port);
      
      os_socket_init();
      
      MainThread *main_thread = new MainThread();
      main_thread->setName("Main Thread");
      main_thread->port = port;
      main_thread->start();

      printf("Press ""Q"" for quit\n");
      int c = getchar();
      while ((c!='q') && (c!='Q'))
      {
        c = getchar();
      }

      os_socket_deinit();
  }
  return 0;
}
